-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore CONSTANTS.VHD 1.10
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
-- Philosophy    see: information/uCore_Philosophy.pdf
--
-- Defining all MicroCore bus-widths and records, as well as the 
-- opcode fields.

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     work.functions.ALL;

PACKAGE constants IS

CONSTANT debug           : std_logic := '0'; -- '1' when debugging
CONSTANT locals          : std_logic := '0'; -- '1' when locals, '0' when simple Kernel
CONSTANT tasks           : std_logic := '0'; -- '1' when task register, '0' when simple Kernel

--------------------------------------------------------------------------
-- microcore busses
--------------------------------------------------------------------------

CONSTANT data_width      : NATURAL := 12; -- data bus width
CONSTANT inst_width      : NATURAL :=  8; -- instruction width
CONSTANT data_addr_width : NATURAL :=  9; -- data memory address width, must be at most one less the data_width because of I/O space
CONSTANT prog_addr_width : NATURAL := 11; -- program memory address width
CONSTANT ds_addr_width   : NATURAL :=  4; -- data stack pointer width
CONSTANT rs_addr_width   : NATURAL :=  8; -- return stack pointer width
CONSTANT interrupts      : NATURAL :=  2; -- maskable interrupt sources

SUBTYPE data_bus      IS std_logic_vector(data_width-1 DOWNTO 0);
SUBTYPE inst_bus      IS std_logic_vector(inst_width-1 DOWNTO 0);
  SUBTYPE inst_group  IS std_logic_vector(inst_width-6 DOWNTO 0);
  SUBTYPE inst_stack  IS std_logic_vector(1 DOWNTO 0);
  SUBTYPE inst_type   IS std_logic_vector(1 DOWNTO 0);

-- to date, synplify does not support SIGNALS in a package. Therefore, the INST-register
-- has to be routed into each entity explicitly as well as the field specific SIGNALs
  
SUBTYPE data_addr     IS std_logic_vector(data_addr_width-1 DOWNTO 0);
SUBTYPE prog_addr     IS std_logic_vector(prog_addr_width-1 DOWNTO 0);
SUBTYPE ds_addr       IS std_logic_vector(ds_addr_width-1 DOWNTO 0);
SUBTYPE rs_addr       IS std_logic_vector(rs_addr_width-1 DOWNTO 0);
SUBTYPE int_bus       IS std_logic_vector(interrupts-1 DOWNTO 0);

--------------------------------------------------------------------------
-- microcore physical addresses and memory mapped registers
--------------------------------------------------------------------------

CONSTANT addr_isr     : std_logic_vector(3 DOWNTO 0) := "0100";
CONSTANT addr_tsr     : std_logic_vector(3 DOWNTO 0) := "1000";

CONSTANT TASK_REGISTER  : data_bus := to_vec(-1, data_width);
CONSTANT DEBUG_REGISTER : data_bus := to_vec(-2, data_width);

--------------------------------------------------------------------------
-- status register
--------------------------------------------------------------------------

CONSTANT status_width : NATURAL :=  8; 
CONSTANT s_c_bit      : NATURAL :=  0;  -- carry bit
CONSTANT s_ie_bit     : NATURAL :=  1;  -- Interrupt Enable bit
CONSTANT s_iis_bit    : NATURAL :=  2;  -- InterruptInService bit 
CONSTANT s_lit_bit    : NATURAL :=  3;  -- LIT bit of the previous instruction
CONSTANT s_n_bit      : NATURAL :=  4;  -- Sign-bit of top data element (TOS or sometimes NOS)
CONSTANT s_z_bit      : NATURAL :=  5;  -- Zero-bit of top data element (TOS or sometimes NOS)

-- these are only active when debug='1'
CONSTANT s_ds_bit     : NATURAL :=  6;  -- DataStack bit - set, when stack empty
CONSTANT s_rs_bit     : NATURAL :=  7;  -- ReturnStack bit - set, when return stack empty

-- tos determines the flags in the status register when the status register is
-- written by STATUS! or IRET.
TYPE status_rec IS RECORD
           flags : std_logic_vector(status_width-1 DOWNTO 0); -- status register flags, see above
           tos   : data_bus;
           write : std_logic; -- rewrite flags from TOS
END RECORD;

TYPE clock_rec IS RECORD
       reset   : std_logic;
       clk     : std_logic;
       clk_en  : std_logic;
       tri_en  : std_logic;  -- delay signal: 3-states uBus at the beginning of every cycle
END RECORD;                  -- only needed when configured for threestate architecture

--------------------------------------------------------------------------
-- microcore record definitions
--------------------------------------------------------------------------
                     
-- Control lines for Data-RAM  
TYPE  dram_port  IS RECORD
               clk   : std_logic;
               write : std_logic;
               addr  : data_addr;
               din   : data_bus;
               dout  : data_bus;
END RECORD;

-- Control lines for Programm-ROM
TYPE  rom_port  IS RECORD
              addr  : prog_addr;
              data  : inst_bus;
END RECORD;

TYPE  io_port  IS RECORD
      trap     : std_logic;  -- hardware PAUSE
      internal : std_logic;
      ioaddr   : std_logic;  -- set for IO-space address
      sel      : std_logic;
      write    : std_logic;
      addr     : data_addr;
      din      : data_bus;
      dout     : data_bus;
END RECORD;

--------------------------------------------------------------------------
-- debug interface somewhat abstracted from Centronics interface
--------------------------------------------------------------------------

CONSTANT in_width         : NATURAL :=  8; -- debugger interface
CONSTANT out_width        : NATURAL :=  1; -- debugger interface

-- Control lines for Program-RAM (debugging)
TYPE  debugrom_port  IS RECORD
              clk   : std_logic;
              write : std_logic;
              addr  : prog_addr;
              din   : inst_bus;
              dout  : inst_bus;
END RECORD;

-- Control lines for the umbilical
TYPE  umbilical  IS RECORD
            init     : std_logic;    -- starts program memory initialisation
            busy     : std_logic;    -- target is busy, don't expect attention

            in_atn   : std_logic;    -- signal from host
            in_str   : std_logic;    -- host placed valid Byte on DIN
            in_ack   : std_logic;    -- to host: last Byte read
            in_data  : std_logic_vector(in_width-1 DOWNTO 0);

            out_atn  : std_logic;    -- indication to host
            out_str  : std_logic;    -- target: one word of data to send
            out_ack  : std_logic;    -- host: out_data has been captured
            out_data : std_logic_vector(out_width-1 DOWNTO 0);
END RECORD;

--------------------------------------------------------------------------
-- op codes
--------------------------------------------------------------------------

--------------------------------------------------------------------------
-- The opcodes' fields are defined in instruction register as follows:
--   7   6   5   4   3   2   1   0
-- |---|---|---|---|---|---|---|---|
-- |   |   |   |   |   |   |   |   |
-- |---|---|---|---|---|---|---|---|
-- |LIT|  TYPE | STACK |    GROUP  |
-- |---|-------|-------|-----------|
--
--------------------------------------------------------------------------
--
--------------------------------------------------------------------------
--                    TYPE
--------------------------------------------------------------------------
-- Code  Name  Action
--  00   BRA   Branches, Calls and Returns
--  01   ALU   Binary and Unary Operators
--  10   MEM   Data-Memory and Register access
--  11   USR   Unused by core, free for user extensions
--------------------------------------------------------------------------
CONSTANT op_BRA    : std_logic_vector(1 DOWNTO 0) := "00";
CONSTANT op_ALU    : std_logic_vector(1 DOWNTO 0) := "01";
CONSTANT op_MEM    : std_logic_vector(1 DOWNTO 0) := "10";
CONSTANT op_USR    : std_logic_vector(1 DOWNTO 0) := "11";

--------------------------------------------------------------------------
--                    STACK
--------------------------------------------------------------------------
-- Code  Name  Action
--  00   NONE  Type dependent
--  01   POP   Stack->NOS->TOS
--  10   PUSH  TOS->NOS->Stack
--  11   BOTH  Type dependent
--------------------------------------------------------------------------
CONSTANT op_NONE   : std_logic_vector(1 DOWNTO 0) := "00";
CONSTANT op_POP    : std_logic_vector(1 DOWNTO 0) := "01";
CONSTANT op_PUSH   : std_logic_vector(1 DOWNTO 0) := "10";
CONSTANT op_BOTH   : std_logic_vector(1 DOWNTO 0) := "11";

--------------------------------------------------------------------------
--                    GROUP
--------------------------------------------------------------------------
--  Code  Binary-Ops  Unary-Ops   Conditions  Registers
--  000   ADD           NOT         NEVER       STATUS
--  001   ADC           SL          ZERO        TOR
--  010   SUB           ASR         NSIGN       RSTACK
--  011   SBC           LSR         NCARRY      LOCAL
--  100   AND           ROR         PAUSE       RSP
--  101   OR            ROL         INT         DSP
--  110   XOR           ZEQU        NZERO       TASK
--  111   NOS           CC          ALWAYS      FLAGS
--------------------------------------------------------------------------
CONSTANT op_ADD    : std_logic_vector(2 DOWNTO 0) := "000";
CONSTANT op_ADC    : std_logic_vector(2 DOWNTO 0) := "001";
CONSTANT op_SUB    : std_logic_vector(2 DOWNTO 0) := "010";
CONSTANT op_SBC    : std_logic_vector(2 DOWNTO 0) := "011";
CONSTANT op_AND    : std_logic_vector(2 DOWNTO 0) := "100";
CONSTANT op_OR     : std_logic_vector(2 DOWNTO 0) := "101";
CONSTANT op_XOR    : std_logic_vector(2 DOWNTO 0) := "110";
CONSTANT op_NOS    : std_logic_vector(2 DOWNTO 0) := "111";

CONSTANT op_NOT    : std_logic_vector(2 DOWNTO 0) := "000";
CONSTANT op_SL     : std_logic_vector(2 DOWNTO 0) := "001";
CONSTANT op_ASR    : std_logic_vector(2 DOWNTO 0) := "010";
CONSTANT op_LSR    : std_logic_vector(2 DOWNTO 0) := "011";
CONSTANT op_ROR    : std_logic_vector(2 DOWNTO 0) := "100";
CONSTANT op_ROL    : std_logic_vector(2 DOWNTO 0) := "101";
CONSTANT op_ZEQU   : std_logic_vector(2 DOWNTO 0) := "110";
CONSTANT op_CC     : std_logic_vector(2 DOWNTO 0) := "111";

CONSTANT op_NEVER  : std_logic_vector(2 DOWNTO 0) := "000";
CONSTANT op_ZERO   : std_logic_vector(2 DOWNTO 0) := "001";
CONSTANT op_NSIGN  : std_logic_vector(2 DOWNTO 0) := "010";
CONSTANT op_NCARRY : std_logic_vector(2 DOWNTO 0) := "011";
CONSTANT op_PAUSE  : std_logic_vector(2 DOWNTO 0) := "100";
CONSTANT op_INT    : std_logic_vector(2 DOWNTO 0) := "101";
CONSTANT op_NZERO  : std_logic_vector(2 DOWNTO 0) := "110";
CONSTANT op_ALWAYS : std_logic_vector(2 DOWNTO 0) := "111";

CONSTANT op_STATUS : std_logic_vector(2 DOWNTO 0) := "000";
CONSTANT op_TOR    : std_logic_vector(2 DOWNTO 0) := "001";
CONSTANT op_RSTACK : std_logic_vector(2 DOWNTO 0) := "010";
CONSTANT op_LOCAL  : std_logic_vector(2 DOWNTO 0) := "011";
CONSTANT op_RSP    : std_logic_vector(2 DOWNTO 0) := "100";
CONSTANT op_DSP    : std_logic_vector(2 DOWNTO 0) := "101";
CONSTANT op_TASK   : std_logic_vector(2 DOWNTO 0) := "110";
CONSTANT op_FLAGS  : std_logic_vector(2 DOWNTO 0) := "111";
CONSTANT op_IE     : std_logic_vector(2 DOWNTO 0) := "111";

CONSTANT NO_OP     : inst_bus := '0' & op_BRA & op_NONE & op_NEVER;
CONSTANT INT_OP    : inst_bus := '0' & op_BRA & op_BOTH & op_INT;
CONSTANT PAUSE_OP  : inst_bus := '0' & op_BRA & op_BOTH & op_PAUSE;
CONSTANT IE_OP     : inst_bus := '0' & op_MEM & op_NONE & op_FLAGS;

END constants;
