-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore DSTACK.VHD 1.10
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
-- Philosophy    see: information/uCore_Philosophy.pdf
--
-- asynchronous Data Stack RAM to be compiled into the FPGA itself
--
-- and
--
-- Data Stack entity consisting of this RAM, the
-- TOS and NOS registers and the stack pointer DSP:

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;

ENTITY dstack_ram IS
PORT (clk      : IN  std_logic;
      write    : IN  std_logic;
      addr     : IN  ds_addr;
      data_in  : IN  data_bus;
      data_out : OUT data_bus);
END dstack_ram;

ARCHITECTURE sim_model OF dstack_ram IS

TYPE ram_stack IS ARRAY (INTEGER RANGE 0 TO 2**ds_addr_width-1) OF data_bus;

SIGNAL ram : ram_stack;

BEGIN

-----------------------------------------------------------------------
-- ram_model: Input data are taken in iff clk rising edge & WE = 1
-----------------------------------------------------------------------
stack_ram: PROCESS(clk)
BEGIN
   IF  rising_edge(clk)  THEN
     IF  write='1'  THEN
         ram(to_int('0' & addr)) <= data_in;
     END IF;
  END IF;
END PROCESS stack_ram;

data_out <= ram(to_int('0' & addr));

END sim_model;

-- ********************************************************************
--
-- Data Stack entity consisting of above RAM, the
-- TOS and NOS registers and the stack pointer DSP:
--
-- ********************************************************************

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     IEEE.std_logic_signed.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;

ENTITY d_stack IS
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic; 
      inst      : IN    inst_bus;
      uBus      : IN    data_bus;
      alu       : IN    data_bus;
      stBus     : INOUT status_rec;
      tos       : OUT   data_bus;
      lit       : OUT   data_bus;
      nos       : OUT   data_bus;
      dsp       : OUT   data_bus);
END d_stack;

ARCHITECTURE rtl OF d_stack IS

COMPONENT dstack_ram
PORT (clk      : IN    std_logic;
      write    : IN    std_logic;
      addr     : IN    ds_addr;
      data_in  : IN    data_bus;
      data_out : OUT   data_bus);
END COMPONENT;

SIGNAL i_lit      : std_logic;
SIGNAL i_type     : inst_type;
SIGNAL i_stack    : inst_stack;
SIGNAL i_group    : inst_group;
SIGNAL nibble     : std_logic_vector(inst_width-2 DOWNTO 0);
SIGNAL s_lit      : std_logic;
SIGNAL s_n        : std_logic;
SIGNAL s_z        : std_logic;
SIGNAL sign       : std_logic;
SIGNAL zero       : std_logic;

SIGNAL tos_reg    : data_bus;
SIGNAL tos_new    : data_bus;
SIGNAL nos_reg    : data_bus;
SIGNAL nos_new    : data_bus;
SIGNAL stack_out  : data_bus;
SIGNAL dsp_reg    : ds_addr;
SIGNAL dsp_new    : ds_addr;
SIGNAL stack_addr : ds_addr;
SIGNAL stack_wr   : std_logic;
SIGNAL status_wr  : std_logic;

BEGIN

lit     <= tos_new;
dsp     <= slice('0', data_width - ds_addr_width) & dsp_reg;
nos     <= nos_reg;
tos     <= tos_reg;

i_lit   <= inst(inst_width-1);
i_type  <= inst(inst_width-2 DOWNTO inst_width-3);
i_stack <= inst(inst_width-4 DOWNTO inst_width-5);
i_group <= inst(inst_width-6 DOWNTO 0);
nibble  <= inst(inst_width-2 DOWNTO 0);

------------------------------------------------------------------------------
-- dstack_control: Generates control lines for data stack entity
-- The opcodes' fields are defined in instruction register as follows:
--   7   6   5   4   3   2   1   0
-- |---|---|---|---|---|---|---|---|
-- |   |   |   |   |   |   |   |   |
-- |---|---|---|---|---|---|---|---|
-- |LIT|  TYPE | STACK |    GROUP  |
-- |---|-------|-------|-----------|
--
-- In the process the op-field are preceded by "i_"
-- The different "op_XXX" are defined in constants.vhdl
-- s_lit is the actual value of LIT in STATUS
-- 
------------------------------------------------------------------------------
dstack_control: PROCESS (nos_reg, tos_reg, i_lit, i_type, i_stack, i_group, stack_out, dsp_reg, uBus, alu, s_lit, nibble)

PROCEDURE push_stack (i : IN data_bus) IS
BEGIN
  nos_new    <= i;
  dsp_new    <= dsp_reg+1;
  stack_addr <= dsp_reg+1;
  stack_wr   <= '1';
END push_stack;

PROCEDURE pop_stack IS
BEGIN
  nos_new    <= stack_out;
  dsp_new    <= dsp_reg-1;
  stack_addr <= dsp_reg;
  stack_wr   <= '0';
END pop_stack;

BEGIN

tos_new    <= tos_reg;
nos_new    <= nos_reg;
dsp_new    <= dsp_reg;
stack_addr <= dsp_reg;
stack_wr   <= '0';
status_wr  <= '0';

IF  i_lit='0'  THEN
   CASE  i_type  IS
     WHEN op_BRA => CASE i_stack IS
                      WHEN op_NONE => CASE i_group IS
                                        WHEN op_INT => pop_stack;
                                                       tos_new <= nos_reg;
                                                       status_wr <= '1';
                                        WHEN op_ZERO | op_NZERO   -- IRET, 0=EXIT, ?EXIT
                                                    => pop_stack;
                                                       tos_new <= nos_reg;
                                        WHEN OTHERS => NULL;
                                      END CASE;
                      WHEN op_POP  => pop_stack;
                                      tos_new <= nos_reg;
                      WHEN op_PUSH => push_stack(tos_reg);
                      WHEN OTHERS  => CASE i_group IS
                                        WHEN op_INT | op_PAUSE  -- interrupt or PAUSE
                                                    => push_stack(tos_reg);
                                                       tos_new <= uBus;
                                        WHEN OTHERS => pop_stack;
                                                       tos_new <= nos_reg;
                                      END CASE;
                      END CASE;
     WHEN op_ALU => tos_new <= alu;
                    CASE i_stack IS
                      WHEN op_NONE => IF  i_group=op_NOS  THEN
                                         nos_new <= tos_reg;
                                      END IF;
                      WHEN op_POP  => pop_stack;
                      WHEN op_PUSH => push_stack(tos_reg);
                      WHEN OTHERS  => NULL;
                    END CASE;
     WHEN op_MEM => CASE i_stack IS
                      WHEN op_NONE => pop_stack;
                                      tos_new <= nos_reg;
                                      CASE  i_group  IS
                                        WHEN op_STATUS => status_wr <= '1';
                                        WHEN op_DSP    => dsp_new <= uBus(ds_addr_width-1 DOWNTO 0);
                                        WHEN OTHERS    => IF  locals='1' AND i_group=op_LOCAL  THEN  -- otherwise, the synthesiser does not
                                                             tos_new <= alu;                         -- handle locals='0' gracefully
                                                          END IF;
                                                          IF  tasks='1' AND i_group=op_TASK  THEN
                                                             tos_new <= alu;
                                                          END IF;
                                      END CASE;
                      WHEN op_POP  => pop_stack;
                                      tos_new <= alu;
                      WHEN op_PUSH => push_stack(uBus);
                                      tos_new <= alu;
                      WHEN OTHERS  => push_stack(tos_reg);
                                      tos_new <= uBus;
                                      IF  locals='1' AND i_group=op_LOCAL  THEN -- otherwise, the synthesiser does not
                                         push_stack(uBus);                      -- handle locals='0' gracefully
                                         tos_new <= alu;
                                      END IF;
                                      IF  tasks='1' AND i_group=op_TASK  THEN
                                         push_stack(uBus);
                                         tos_new <= alu;
                                      END IF;
                    END CASE;
     WHEN OTHERS => NULL;
   END CASE;
ELSE -- i_lit='1'
   tos_new <= tos_reg(data_width-8 DOWNTO 0) & nibble;
   IF  s_lit='0'  THEN
      push_stack(tos_reg);
      tos_new <= slice(nibble(6), data_width-7) & nibble;
   END IF;
END IF;

END PROCESS dstack_control;

stack_registers: PROCESS(clk, reset)
BEGIN
   IF  reset='1'  THEN
      tos_reg <= (OTHERS => '0');
      nos_reg <= (OTHERS => '0');
      dsp_reg <= (OTHERS => '0');
   ELSIF  rising_edge(clk)  THEN
     IF  clk_en='1'  THEN
        tos_reg <= tos_new;
        nos_reg <= nos_new;
        dsp_reg <= dsp_new;
     END IF;
   END IF;
END PROCESS stack_registers;

------------------------------------------------------------------------------
-- sign_and_zero: Fetch the tos status
--             STATUS 
--   7   6   5   4   3   2   1   0
-- |---|---|---|---|---|---|---|---|
-- |   |   |   |   |   |   |   |   |
-- |---|---|---|---|---|---|---|---|
-- |   |   | Z | N |LIT|IIS|IE | C |
-- |---|---|---|---|---|---|---|---|
--
-- (a) reset = 1: N, Z and LIT all 0 
-- (b) clk valid & status.write = 1: fetch flags from tos
-- (c) clk valid & LIT = 0: signals sign and zero determine N and Z flags
-- (d) clk valid & s_lit = 0: signals sign and zero determine N and Z flags
------------------------------------------------------------------------------

sign <= tos_reg(data_width-1);
zero <= '1' WHEN  tos_reg=slice('0', data_width)  ELSE  '0';

sign_and_zero: PROCESS(clk, reset)
BEGIN
   IF  reset='1'  THEN
      s_n <= '0';
      s_z <= '0';
      s_lit <= '0';
   ELSIF  rising_edge(clk)  THEN
     IF  clk_en='1'  THEN
       s_lit <= i_lit;
       IF  stBus.write='1'  THEN
          s_n   <= stBus.tos(s_n_bit);
          s_z   <= stBus.tos(s_z_bit);
          s_lit <= stBus.tos(s_lit_bit);
          IF  stBus.tos(s_lit_bit)='0'  THEN
             s_n <= sign;
             s_z <= zero;
          END IF;
       ELSIF  s_lit='0'  THEN
          s_n <= sign;
          s_z <= zero;
       END IF;
     END IF;
   END IF;
END PROCESS sign_and_zero;

stBus.flags(s_lit_bit) <= s_lit;
stBus.flags(s_n_bit)   <= sign  WHEN  s_lit='0'  ELSE  s_n;
stBus.flags(s_z_bit)   <= zero  WHEN  s_lit='0'  ELSE  s_z;
stBus.write            <= status_wr;
stBus.tos              <= tos_reg;

-- map RAM component of data_satck
data_stack: dstack_ram
PORT MAP(clk      => clk,
         write    => stack_wr,
         addr     => stack_addr,
         data_in  => nos_reg, 
         data_out => stack_out);

END rtl;

