-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore ROM_RAM_CORE.VHD 1.10
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
-- Philosophy    see: information/uCore_Philosophy.pdf
--
-- Top level entity with external program ROM and 
-- external data/return-stack RAM. This file should
-- be edited for technology specific additions like e.g.
-- pad assignments.

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;

ENTITY core IS
PORT (reset       : IN    std_logic;
      clk         : IN    std_logic;
      clk_en      : IN    std_logic;
      delay       : INOUT std_logic; -- unused pad, serves to generate tri_en signal
      iosel       : OUT   std_logic;
      iotrap      : IN    std_logic;
      iowrite     : OUT   std_logic;
      ioaddr      : OUT   data_addr;
      iodout      : OUT   data_bus;
      iodin       : IN    data_bus;
      romaddr     : OUT   prog_addr;
      instruction : IN    inst_bus;
      ramwr       : OUT   std_logic;
      ramaddr     : OUT   data_addr;
      ramdin      : OUT   data_bus;
      ramdout     : IN    data_bus;
      int_in      : IN    int_bus);
END core;

ARCHITECTURE technology OF core IS

COMPONENT microcore
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic;
      tri_en    : IN    std_logic;  
      int_in    : IN    int_bus;
      dram      : INOUT dram_port;
      rom       : INOUT rom_port;
      ioBus     : INOUT io_port);
END COMPONENT;

COMPONENT data_ram
PORT (dram : INOUT dram_port);
END COMPONENT;

COMPONENT code_rom
PORT (rom : INOUT rom_port);
END COMPONENT;

ATTRIBUTE syn_keep : BOOLEAN;

SIGNAL tri_en    : std_logic;
SIGNAL mclk      : std_logic; 
SIGNAL delay_en  : std_logic;  ATTRIBUTE syn_keep OF delay_en : SIGNAL IS true;

SIGNAL dram      : dram_port;
SIGNAL rom       : rom_port;
SIGNAL ioBus     : io_port;

BEGIN

mclk <= clk;

-----------------------------------------------------------------------
--  TRI_EN using a pad with in- and output buffer as delay element
-- 'Z'-states uBus for 20 ns to prevent cross-currents
-- To force the synthesizer to generate such logic, the pad is defined
-- as a tri-state pad (INOUT) that is continuously enabled.
-----------------------------------------------------------------------

delay_en <= '1';
delay    <= mclk AFTER 20 ns WHEN delay_en='1' ELSE 'Z';

tristate_holdoff: PROCESS (delay, mclk)
BEGIN
  IF  delay='1'  THEN
     tri_en <= '1';
  ELSIF  rising_edge(mclk)  THEN
     IF  clk_en='1'  THEN
        tri_en <= '0';
     END IF;
  END IF;
END PROCESS tristate_holdoff;

uCore: microcore
PORT MAP(reset  => reset,
         clk    => mclk,
         clk_en => clk_en,
         tri_en => tri_en,
         int_in => int_in,
         dram   => dram,
         rom    => rom,
         ioBus  => ioBus);

--data_memory: data_ram PORT MAP (dram => dram);
ramwr     <= dram.write;
ramaddr   <= dram.addr;
ramdin    <= dram.din;
dram.dout <= ramdout;
dram.clk  <= clk;

--prog_memory: code_rom PORT MAP (rom => rom);
romaddr   <= rom.addr;
rom.data  <= instruction;

-----------------------------------------------------------------------
--  io interface
-----------------------------------------------------------------------

iosel       <= ioBus.sel;
ioBus.trap  <= iotrap AND ioBus.sel;
iowrite     <= ioBus.write;
ioaddr      <= ioBus.addr;
iodout      <= ioBus.dout;
ioBus.din   <= iodin;

END technology;