-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore UCORE.VHD 1.10
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
-- Philosophy    see: information/uCore_Philosophy.pdf
--
-- Structural integration of all uCore MicroCells.

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     IEEE.std_logic_signed.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;

ENTITY microcore IS
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic;
      tri_en    : IN    std_logic;  
      int_in    : IN    int_bus;
      dram      : INOUT dram_port;
      rom       : INOUT rom_port;
      ioBus     : INOUT io_port);
END microcore;

ARCHITECTURE structural OF microcore IS

COMPONENT d_stack
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic; 
      inst      : IN    inst_bus;
      uBus      : IN    data_bus;
      alu       : IN    data_bus;
      stBus     : INOUT status_rec;
      tos       : OUT   data_bus;
      lit       : OUT   data_bus;
      nos       : OUT   data_bus;
      dsp       : OUT   data_bus);
END COMPONENT;

COMPONENT alu
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic; 
      tri_en    : IN    std_logic;  
      inst      : IN    inst_bus;
      uBus      : IN    data_bus;
      tos       : IN    data_bus;
      lit       : IN    data_bus;
      nos       : IN    data_bus;
      pc        : IN    data_bus;
      rsp       : IN    data_bus;
      task      : IN    data_bus;
      stBus     : INOUT status_rec;
      memaddr   : OUT   data_bus;
      alu       : OUT   data_bus);
END COMPONENT;

COMPONENT r_stack
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic; 
      inst      : IN    inst_bus;
      condition : IN    std_logic;
      uBus      : IN    data_bus;
      ioBus     : IN    io_port;
      memaddr   : IN    data_bus;
      nos       : IN    data_bus;
      alu       : IN    data_bus;
      pc        : IN    data_bus;
      dram      : INOUT dram_port;
      rsp       : OUT   data_bus);
END COMPONENT;

COMPONENT sequencer
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic; 
      int       : IN    std_logic;
      ioBus     : IN    io_port;
      stBus     : IN    status_rec;
      uBus      : IN    data_bus;
      inst      : OUT   inst_bus;
      condition : OUT   std_logic;
      pc        : OUT   data_bus;
      rom       : INOUT rom_port);
END COMPONENT;

COMPONENT interrupt
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic; 
      int_in    : IN    int_bus;
      inst      : IN    inst_bus;
      stBus     : INOUT status_rec;
      int       : OUT   std_logic;
      flags     : OUT   data_bus);
END COMPONENT;

COMPONENT microbus
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic; 
      tri_en    : IN    std_logic;  
      inst      : IN    inst_bus;
      stBus     : IN    status_rec;
      tos       : IN    data_bus;
      nos       : IN    data_bus;
      alu       : IN    data_bus;
      dsp       : IN    data_bus;
      rsp       : IN    data_bus;
      dram      : IN    dram_port;
      flags     : IN    data_bus;
      task      : IN    data_bus;
      ioBus     : IN    io_port;
      uBus      : INOUT data_bus);
END COMPONENT;

COMPONENT io 
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic; 
      inst      : IN    inst_bus;
      memaddr   : IN    data_bus;
      nos       : IN    data_bus;
      ioBus     : INOUT io_port;
      task      : OUT   data_bus);
END COMPONENT;
 
-----------------------------------------------------------------------
--  core's signals
-----------------------------------------------------------------------

SIGNAL uBus        : data_bus;
SIGNAL alu_out     : data_bus;
SIGNAL memaddr     : data_bus;
SIGNAL tos         : data_bus;
SIGNAL lit         : data_bus;
SIGNAL nos         : data_bus;
SIGNAL pc          : data_bus;
SIGNAL flags       : data_bus;
SIGNAL dsp         : data_bus;
SIGNAL rsp         : data_bus;
SIGNAL task        : data_bus;
SIGNAL stBus       : status_rec;
SIGNAL inst        : inst_bus;
SIGNAL trap_clk_en : std_logic; 
SIGNAL condition   : std_logic;
SIGNAL internal    : std_logic; -- true when addressing internal registers in IO space
SIGNAL ioaddr      : std_logic; -- address is for I/O, not data memory or rstack
SIGNAL int         : std_logic;

BEGIN

trap_clk_en <= clk_en AND NOT ioBus.trap;

-----------------------------------------------------------------------
-- stack empty status bits needed only for the debugger
-----------------------------------------------------------------------
stBus.flags(s_ds_bit) <= '1'  WHEN  dsp(ds_addr_width-1 DOWNTO 0)=0  AND debug='1'  ELSE '0';
stBus.flags(s_rs_bit) <= '1'  WHEN  rsp(rs_addr_width-1 DOWNTO 0)=-1 AND debug='1'  ELSE '0';

data_stack: d_stack
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => trap_clk_en,
         inst      => inst,
         uBus      => uBus,
         alu       => alu_out,
         stBus     => stBus,
         tos       => tos,
         lit       => lit,
         nos       => nos,
         dsp       => dsp);

arithmetic: alu
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => trap_clk_en,
         tri_en    => tri_en,
         inst      => inst,
         uBus      => uBus,
         tos       => tos,
         lit       => lit,
         nos       => nos,
         pc        => pc,
         rsp       => rsp,
         task      => task,
         stBus     => stBus,
         memaddr   => memaddr,
         alu       => alu_out);

return_stack: r_stack
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => trap_clk_en,
         inst      => inst,
         condition => condition,
         uBus      => uBus,
         ioBus     => ioBus,
         memaddr   => memaddr,
         nos       => nos,
         alu       => alu_out,
         pc        => pc,
         dram      => dram,
         rsp       => rsp);

program_sequencer: sequencer
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => trap_clk_en,
         int       => int,
         ioBus     => ioBus,
         stBus     => stBus,
         uBus      => uBus,
         inst      => inst,
         condition => condition,
         pc        => pc,
         rom       => rom);

int_unit: interrupt
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => trap_clk_en,
         int_in    => int_in,
         inst      => inst,
         stBus     => stBus,
         int       => int,
         flags     => flags);

microcore_bus: microbus
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => trap_clk_en,
         tri_en    => tri_en,
         inst      => inst,
         stBus     => stBus,
         tos       => tos,
         nos       => nos,
         alu       => alu_out,
         dsp       => dsp,
         rsp       => rsp,
         dram      => dram,
         flags     => flags,
         task      => task,
         ioBus     => ioBus,
         uBus      => uBus);

uCore_io: io 
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => trap_clk_en,
         inst      => inst,
         memaddr   => memaddr,
         nos       => nos,
         ioBus     => ioBus,
         task      => task);

END structural;
